/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.crop.type;

import java.io.IOException;
import java.util.HashMap;
import nl.cloudfarming.client.crop.model.*;
import org.netbeans.api.project.Project;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.util.Exceptions;
import org.openide.util.Lookup;
import org.openide.util.lookup.ServiceProvider;

/**
 * Service for getting and adding crops to a certain project
 *
 * @author Merijn Zengers
 */
@ServiceProvider(service = CropService.class)
public class CropServiceImpl implements CropService {

    protected static final String CROP_DIR = "crops";
    protected static final String CROP_PREFIX = ".crp";
    protected static final String NEW_CROP_FILE_NAME = "new";
    private CropManagerImpl cropManagerImpl;
    private HashMap<Project, CropsStorageDataObject> projects;
    private CropsLoader loader;

    public CropServiceImpl() {
        projects = new HashMap<>();
        loader = Lookup.getDefault().lookup(CropsLoader.class);
    }

    @Override
    public CropsStorage getCropsStorage(Project project) throws DataObjectNotFoundException, IOException {
        return getCropsStorageDataObject(project).getCropsStorage();
    }

    @Override
    public void addCrop(Project project, Crop crop) throws DataObjectNotFoundException, IOException {
        CropsStorageDataObject cropsStorageDataObject = getCropsStorageDataObject(project);

        if (!cropsStorageDataObject.getCropsStorage().getCrops().contains(crop)) {
            cropsStorageDataObject.getCropsStorage().addCrop(crop);
            cropsStorageDataObject.save();
        }
    }

    @Override
    public void addStockProject(Project project) {
        projects.put(project, null);
        loader.loadCrops(project);
    }

    @Override
    public void removeStockProject(Project project) {
        projects.remove(project);
    }

    @Override
    public void addCrop(Crop crop) throws DataObjectNotFoundException, IOException {
        for (Project project : projects.keySet()) {
            addCrop(project, crop);
        }
    }

    @Override
    public CropsStorageDataObject getCropsStorageDataObject(Project project) throws DataObjectNotFoundException, IOException {
        if(projects.containsKey(project) && projects.get(project) != null){
            return projects.get(project);
        }
        FileObject cropFolder = getCropFolder(project.getProjectDirectory(), true);
        for (FileObject file : cropFolder.getChildren()) {
            DataObject found = DataObject.find(file);
            if (found instanceof CropsStorageDataObject) {
                projects.put(project, (CropsStorageDataObject)found);
                return (CropsStorageDataObject) found;
            }
        }
        return null;
    }

    protected CropManagerImpl getCropManager() {
        if (cropManagerImpl == null) {
            cropManagerImpl = new CropManagerImpl();
        }
        return cropManagerImpl;
    }

    /**
     * Method to get the crop folder containing crops
     *
     * @param projectDir the projectDir the crop directory should reside in
     * @param create if not found create the folder
     * @return the found or created FileObject
     */
    public FileObject getCropFolder(FileObject projectDir, boolean create) {
        FileObject result = projectDir.getFileObject(CROP_DIR);
        if (result == null && create) {
            try {
                result = projectDir.createFolder(CROP_DIR);
            } catch (IOException ioe) {
                Exceptions.printStackTrace(ioe);
            }
        }
        return result;
    }
}
